@@ -35,7 +35,7 @@ gfu_common_checksum_format(const gchar *checksum)
 	return g_strdup_printf("%s(%s)", checksum_type, checksum);
 }
 
-const gchar *
+gchar *
 gfu_common_seconds_to_string(guint64 seconds)
 {
 	guint64 minutes, hours;
@@ -182,7 +182,7 @@ gfu_common_release_flag_to_string(FwupdReleaseFlags release_flag)
 	return fwupd_release_flag_to_string(release_flag);
 }
 
-const gchar *
+gchar *
 gfu_common_release_flags_to_strings(guint64 flags)
 {
 	g_autoptr(GString) str = g_string_new(NULL);
@@ -306,10 +306,6 @@ gfu_common_device_flags_to_string(guint64 device_flag)
 		/* TRANSLATORS: Device is updatable in this or any other mode */
 		return _("Updatable");
 	}
-	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_OFFLINE) {
-		/* TRANSLATORS: Update can only be done from offline mode */
-		return _("Update requires a reboot");
-	}
 	if (device_flag == FWUPD_DEVICE_FLAG_REQUIRE_AC) {
 		/* TRANSLATORS: Must be plugged in to an outlet */
 		return _("System requires external power source");
@@ -342,14 +338,6 @@ gfu_common_device_flags_to_string(guint64 device_flag)
 		/* TRANSLATORS: User has been notified */
 		return _("User has been notified");
 	}
-	if (device_flag == FWUPD_DEVICE_FLAG_USE_RUNTIME_VERSION) {
-		/* skip */
-		return NULL;
-	}
-	if (device_flag == FWUPD_DEVICE_FLAG_INSTALL_PARENT_FIRST) {
-		/* TRANSLATORS: Install composite firmware on the parent before the child */
-		return _("Install to parent device first");
-	}
 	if (device_flag == FWUPD_DEVICE_FLAG_IS_BOOTLOADER) {
 		/* TRANSLATORS: Is currently in bootloader mode */
 		return _("Is in bootloader mode");
@@ -358,10 +346,6 @@ gfu_common_device_flags_to_string(guint64 device_flag)
 		/* TRANSLATORS: The hardware is waiting to be replugged */
 		return _("Hardware is waiting to be replugged");
 	}
-	if (device_flag == FWUPD_DEVICE_FLAG_IGNORE_VALIDATION) {
-		/* TRANSLATORS: Ignore validation safety checks when flashing this device */
-		return _("Ignore validation safety checks");
-	}
 	if (device_flag == FWUPD_DEVICE_FLAG_ANOTHER_WRITE_REQUIRED) {
 		/* skip */
 		return NULL;
@@ -402,10 +386,6 @@ gfu_common_device_flags_to_string(guint64 device_flag)
 		/* TRANSLATORS: Device does not jump directly to the newest */
 		return _("Device installs all intermediate releases");
 	}
-	if (device_flag == FWUPD_DEVICE_FLAG_SKIPS_RESTART) {
-		/* TRANSLATORS: Device does not restart and requires a manual action */
-		return _("Device skips restart");
-	}
 	if (device_flag == FWUPD_DEVICE_FLAG_HAS_MULTIPLE_BRANCHES) {
 		/* TRANSLATORS: the stream can be provided by different vendors */
 		return _("Device supports switching to a different stream of firmware");
@@ -474,8 +454,6 @@ gfu_common_device_icon_from_flag(FwupdDeviceFlags device_flag)
 		return "drive-harddisk-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_UPDATABLE)
 		return "software-update-available-symbolic";
-	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_OFFLINE)
-		return "network-offline-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_REQUIRE_AC)
 		return "battery-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_LOCKED)
@@ -492,16 +470,10 @@ gfu_common_device_icon_from_flag(FwupdDeviceFlags device_flag)
 		return "task-due-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_NOTIFIED)
 		return "task-due-symbolic";
-	if (device_flag == FWUPD_DEVICE_FLAG_USE_RUNTIME_VERSION)
-		return "system-run-symbolic";
-	if (device_flag == FWUPD_DEVICE_FLAG_INSTALL_PARENT_FIRST)
-		return "system-software-install-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_IS_BOOTLOADER)
 		return "computer-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG)
 		return "battery-low-symbolic";
-	if (device_flag == FWUPD_DEVICE_FLAG_IGNORE_VALIDATION)
-		return "dialog-error-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_ANOTHER_WRITE_REQUIRED)
 		return "media-floppy-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION)
@@ -518,8 +490,6 @@ gfu_common_device_icon_from_flag(FwupdDeviceFlags device_flag)
 		return "emblem-important-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_INSTALL_ALL_RELEASES)
 		return "media-skip-forward-symbolic";
-	if (device_flag == FWUPD_DEVICE_FLAG_SKIPS_RESTART)
-		return "process-stop-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_HAS_MULTIPLE_BRANCHES)
 		return "emblem-shared-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_BACKUP_BEFORE_INSTALL)
@@ -542,10 +512,6 @@ gfu_common_device_icon_from_flag(FwupdDeviceFlags device_flag)
 		return "media-playlist-repeat-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_EMULATION_TAG)
 		return "media-playlist-repeat-song-symbolic";
-#if FWUPD_CHECK_VERSION(1, 9, 3)
-	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_EXPLICIT_UPDATES)
-		return "user-info-symbolic";
-#endif
 	if (device_flag == FWUPD_DEVICE_FLAG_UNKNOWN)
 		return "unknown-symbolic";
 	return NULL;
