@@ -17,10 +17,17 @@
 #include "gfu-device-row.h"
 #include "gfu-release-row.h"
 
+#define GFU_MAIN_REPORT_DEVICE_LIST_INITIAL_DELAY  5  /* s */
+#define GFU_MAIN_REPORT_DEVICE_LIST_INTERVAL_DELAY 30 /* days */
+
+/* should this come from FwupdRemote? */
+#define GFU_LVFS_PRIVACY_POLICY_URL "https://lvfs.readthedocs.io/en/latest/privacy.html"
+
 typedef void (*GfuMainDialogFunc)(gpointer user_data);
 
 typedef struct {
 	AdwApplication *application;
+	GSettings *settings;
 	GtkBuilder *builder;
 	GCancellable *cancellable;
 	FwupdClient *client;
@@ -32,25 +39,21 @@ typedef struct {
 	GtkWindow *dialog_current; /* no-ref */
 	GfuMainDialogFunc dialog_func;
 	GPtrArray *post_requests;
+	GHashTable *report_metadata;
+	gchar *report_devices_payload;
+	GPtrArray *devices;
+	FwupdRemote *remote_lvfs;
 } GfuMain;
 
 static void
 gfu_main_release_row_button_clicked_cb(GfuReleaseRow *row, GfuMain *self);
 static void
+gfu_main_refresh_release(GfuMain *self);
+static void
 gfu_main_refresh_releases(GfuMain *self);
 static void
 gfu_main_process_dialogs(GfuMain *self);
 
-/* GTK helper functions */
-
-static void
-gfu_main_list_box_remove_all(GtkListBox *container)
-{
-	GtkWidget *child;
-	while ((child = gtk_widget_get_last_child(GTK_WIDGET(container))) != NULL)
-		gtk_list_box_remove(container, child);
-}
-
 /* this takes ownership */
 static void
 gfu_main_add_dialog(GfuMain *self, GtkWindow *dialog, gboolean process_now)
@@ -162,17 +165,37 @@ gfu_main_guid_instance_id_new(const gchar *instance_id, const gchar *guid)
 	return row;
 }
 
+static GPtrArray *
+gfu_main_get_secret_children(GtkWidget *w)
+{
+	GPtrArray *children = g_object_get_data(G_OBJECT(w), "children");
+	if (children != NULL)
+		return children;
+	children = g_ptr_array_new();
+	g_object_set_data_full(G_OBJECT(w),
+			       "children",
+			       children,
+			       (GDestroyNotify)g_ptr_array_unref);
+	return children;
+}
+
 static void
 gfu_main_update_guids(GfuMain *self, GPtrArray *guids, GPtrArray *instance_ids)
 {
+	GPtrArray *children;
 	GtkWidget *w;
 	g_autoptr(GHashTable) guid_hash = NULL;
 
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "row_guids"));
 	gtk_widget_set_visible(w, guids->len > 0);
 
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "listbox_guids"));
-	gfu_main_list_box_remove_all(GTK_LIST_BOX(w));
+	/* we have to do this hack as adw_expander_row_remove_all() does not exist */
+	children = gfu_main_get_secret_children(w);
+	for (guint i = 0; i < children->len; i++) {
+		GtkWidget *row = g_ptr_array_index(children, i);
+		adw_expander_row_remove(ADW_EXPANDER_ROW(w), row);
+	}
+	g_ptr_array_set_size(children, 0);
 
 	/* instance IDs */
 	guid_hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
@@ -181,7 +204,8 @@ gfu_main_update_guids(GfuMain *self, GPtrArray *guids, GPtrArray *instance_ids)
 		g_autofree gchar *guid = fwupd_guid_hash_string(instance_id);
 		GtkWidget *row = gfu_main_guid_instance_id_new(instance_id, guid);
 		g_hash_table_add(guid_hash, g_steal_pointer(&guid));
-		gtk_list_box_insert(GTK_LIST_BOX(w), row, -1);
+		adw_expander_row_add_row(ADW_EXPANDER_ROW(w), row);
+		g_ptr_array_add(children, row);
 	}
 
 	/* GUID fallback */
@@ -190,63 +214,54 @@ gfu_main_update_guids(GfuMain *self, GPtrArray *guids, GPtrArray *instance_ids)
 		GtkWidget *row;
 		if (g_hash_table_contains(guid_hash, guid))
 			continue;
+
 		row = gfu_main_guid_instance_id_new(NULL, guid);
-		gtk_list_box_insert(GTK_LIST_BOX(w), row, -1);
+		adw_expander_row_add_row(ADW_EXPANDER_ROW(w), row);
+		g_ptr_array_add(children, row);
 	}
 }
 
 static void
 gfu_main_set_device_flags(GfuMain *self, guint64 flags)
 {
-	GtkWidget *icon, *label;
-	GtkGrid *w;
-	gint count = 0;
-	g_autoptr(GString) flag = g_string_new(NULL);
+	GtkWidget *w =
+	    GTK_WIDGET(gtk_builder_get_object(self->builder, "label_device_flags_title"));
+	GPtrArray *children = gfu_main_get_secret_children(w);
 
-	/* clear the grid */
-	w = GTK_GRID(gtk_builder_get_object(self->builder, "grid_device_flags"));
-	gtk_grid_remove_column(w, 0);
-	gtk_grid_remove_column(w, 0);
-	gtk_grid_insert_column(w, 0);
-	gtk_grid_insert_column(w, 0);
+	gtk_widget_set_visible(GTK_WIDGET(w), FALSE);
+
+	/* we have to do this hack as adw_expander_row_remove_all() does not exist */
+	for (guint i = 0; i < children->len; i++) {
+		GtkWidget *row = g_ptr_array_index(children, i);
+		adw_expander_row_remove(ADW_EXPANDER_ROW(w), row);
+	}
+	g_ptr_array_set_size(children, 0);
 
 	/* iterate through flags */
 	for (guint j = 0; j < 64; j++) {
+		GtkWidget *row;
+		GtkWidget *icon;
 		const gchar *flag_tmp;
+		FwupdDeviceFlags flag = (guint64)1 << j;
+
 		/* if flag is not set, skip */
-		if ((flags & ((guint64)1 << j)) == 0)
+		if ((flags & flag) == 0)
 			continue;
-		/* else, add a row for this flag */
-		g_string_set_size(flag, 0);
-		flag_tmp = gfu_common_device_flags_to_string((guint64)1 << j);
+
+		/* some are not interesting */
+		flag_tmp = gfu_common_device_flags_to_string(flag);
 		if (flag_tmp == NULL)
 			continue;
-		g_string_assign(flag, flag_tmp);
-		gtk_grid_insert_row(w, count);
 
-		icon =
-		    gtk_image_new_from_icon_name(gfu_common_device_icon_from_flag((guint64)1 << j));
-		gtk_widget_set_visible(icon, TRUE);
-		gtk_grid_attach(w, icon, 0, count, 1, 1);
+		/* add row with icon */
+		row = adw_action_row_new();
+		adw_preferences_row_set_title(ADW_PREFERENCES_ROW(row), flag_tmp);
+		icon = gtk_image_new_from_icon_name(gfu_common_device_icon_from_flag(flag));
+		adw_action_row_add_prefix(ADW_ACTION_ROW(row), icon);
+		adw_expander_row_add_row(ADW_EXPANDER_ROW(w), row);
 
-		label = gtk_label_new(flag->str);
-		gtk_label_set_xalign(GTK_LABEL(label), 0);
-		gtk_widget_set_visible(label, TRUE);
-		gtk_label_set_wrap(GTK_LABEL(label), TRUE);
-		gtk_grid_attach(w, label, 1, count, 1, 1);
-		count++;
-	}
-	if (count == 0) {
-		/* no flags were added */
-		gtk_widget_set_visible(GTK_WIDGET(w), FALSE);
-		gtk_widget_set_visible(
-		    GTK_WIDGET(gtk_builder_get_object(self->builder, "label_device_flags_title")),
-		    FALSE);
-	} else {
+		g_ptr_array_add(children, row);
 		gtk_widget_set_visible(GTK_WIDGET(w), TRUE);
-		gtk_widget_set_visible(
-		    GTK_WIDGET(gtk_builder_get_object(self->builder, "label_device_flags_title")),
-		    TRUE);
 	}
 }
 
@@ -311,18 +326,14 @@ gfu_main_device_problem_to_string(GfuMain *self, FwupdDeviceProblem problem)
 		/* TRANSLATORS: lid means "laptop top cover" */
 		return g_strdup(_("Device cannot be used while the lid is closed"));
 	}
-#if FWUPD_CHECK_VERSION(1, 9, 1)
 	if (problem == FWUPD_DEVICE_PROBLEM_IN_USE) {
 		/* TRANSLATORS: device cannot be interrupted, for instance taking a phone call */
 		return g_strdup(_("Device is in use"));
 	}
-#endif
-#if FWUPD_CHECK_VERSION(1, 9, 6)
 	if (problem == FWUPD_DEVICE_PROBLEM_DISPLAY_REQUIRED) {
 		/* TRANSLATORS: device cannot be interrupted, for instance taking a phone call */
 		return g_strdup(_("Device cannot be used while there are no displays plugged in"));
 	}
-#endif
 	return NULL;
 }
 
@@ -364,6 +375,7 @@ gfu_main_refresh_device_cb(gpointer user_data)
 	GtkWidget *split_view_devices =
 	    GTK_WIDGET(gtk_builder_get_object(self->builder, "split_view_devices"));
 	GPtrArray *vendor_ids = NULL;
+	g_autofree gchar *install_duration = NULL;
 	g_autofree gchar *problems_str = NULL;
 	g_autoptr(GString) attr = g_string_new(NULL);
 	g_autoptr(GString) version = NULL;
@@ -428,11 +440,9 @@ gfu_main_refresh_device_cb(gpointer user_data)
 	gfu_main_set_label(self,
 			   "label_device_flashes_left",
 			   g_strcmp0(attr->str, "0") ? attr->str : NULL);
-
-	gfu_main_set_label(
-	    self,
-	    "label_device_install_duration",
-	    gfu_common_seconds_to_string(fwupd_device_get_install_duration(self->device)));
+	install_duration =
+	    gfu_common_seconds_to_string(fwupd_device_get_install_duration(self->device));
+	gfu_main_set_label(self, "label_device_install_duration", install_duration);
 
 	gfu_main_update_guids(self,
 			      fwupd_device_get_guids(self->device),
@@ -547,12 +557,51 @@ gfu_main_reboot_restart_response_cb(GtkDialog *dialog, int response_id, GfuMain
 	}
 }
 
+static void
+gfu_main_build_report_devices_payload(GfuMain *self)
+{
+#if FWUPD_CHECK_VERSION(1, 9, 20)
+	g_autoptr(GError) error_local = NULL;
+
+	g_clear_pointer(&self->report_devices_payload, g_free);
+
+	if (self->remote_lvfs == NULL) {
+		g_debug("no LVFS remote");
+		return;
+	}
+	if (self->report_metadata == NULL) {
+		g_debug("no report metadata");
+		return;
+	}
+	if (!fwupd_remote_has_flag(self->remote_lvfs, FWUPD_REMOTE_FLAG_ENABLED)) {
+		g_debug("%s not enabled, skipping", fwupd_remote_get_id(self->remote_lvfs));
+		return;
+	}
+	if (fwupd_remote_get_report_uri(self->remote_lvfs) == NULL) {
+		g_debug("no report URL for %s, skipping", fwupd_remote_get_id(self->remote_lvfs));
+		return;
+	}
+
+	/* lets give it a go */
+	g_free(self->report_devices_payload);
+	self->report_devices_payload = fwupd_client_build_report_devices(self->client,
+									 self->devices,
+									 self->report_metadata,
+									 &error_local);
+	if (self->report_devices_payload == NULL) {
+		g_debug("failed to build report for %s: %s",
+			fwupd_remote_get_id(self->remote_lvfs),
+			error_local->message);
+		return;
+	}
+#endif
+}
+
 static void
 gfu_main_update_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 {
 	GfuMain *self = (GfuMain *)user_data;
 	GtkWidget *w;
-	gboolean disabled_lvfs_remote = FALSE;
 	gboolean enabled_any_download_remote = FALSE;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GPtrArray) remotes = NULL;
@@ -565,6 +614,11 @@ gfu_main_update_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 	}
 	for (guint i = 0; i < remotes->len; i++) {
 		FwupdRemote *remote = g_ptr_array_index(remotes, i);
+
+		/* rather than getting by ID */
+		if (g_strcmp0(fwupd_remote_get_id(remote), "lvfs") == 0)
+			g_set_object(&self->remote_lvfs, remote);
+
 #if FWUPD_CHECK_VERSION(2, 0, 0)
 		/* if another repository is turned on providing metadata */
 		g_debug("%s is %s",
@@ -575,9 +629,6 @@ gfu_main_update_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 			if (fwupd_remote_get_kind(remote) == FWUPD_REMOTE_KIND_DOWNLOAD)
 				enabled_any_download_remote = TRUE;
 			/* lvfs is present and disabled */
-		} else {
-			if (g_strcmp0(fwupd_remote_get_id(remote), "lvfs") == 0)
-				disabled_lvfs_remote = TRUE;
 		}
 #else
 		/* if another repository is turned on providing metadata */
@@ -587,22 +638,31 @@ gfu_main_update_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 		if (fwupd_remote_get_enabled(remote)) {
 			if (fwupd_remote_get_kind(remote) == FWUPD_REMOTE_KIND_DOWNLOAD)
 				enabled_any_download_remote = TRUE;
-
-			/* lvfs is present and disabled */
-		} else {
-			if (g_strcmp0(fwupd_remote_get_id(remote), "lvfs") == 0)
-				disabled_lvfs_remote = TRUE;
 		}
 #endif
 	}
 
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "infobar_enable_lvfs"));
-	if (disabled_lvfs_remote && !enabled_any_download_remote) {
-		gtk_widget_set_visible(w, TRUE);
-		gtk_info_bar_set_revealed(GTK_INFO_BAR(w), TRUE);
+	if (self->remote_lvfs != NULL &&
+	    !fwupd_remote_has_flag(self->remote_lvfs, FWUPD_REMOTE_FLAG_ENABLED) &&
+	    !enabled_any_download_remote) {
+		w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_enable_lvfs"));
+		adw_banner_set_revealed(ADW_BANNER(w), TRUE);
+		w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_refresh_lvfs"));
+		adw_banner_set_revealed(ADW_BANNER(w), FALSE);
+	} else if (self->remote_lvfs != NULL && fwupd_remote_needs_refresh(self->remote_lvfs)) {
+		w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_enable_lvfs"));
+		adw_banner_set_revealed(ADW_BANNER(w), FALSE);
+		w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_refresh_lvfs"));
+		adw_banner_set_revealed(ADW_BANNER(w), TRUE);
 	} else {
-		gtk_widget_set_visible(w, FALSE);
+		w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_enable_lvfs"));
+		adw_banner_set_revealed(ADW_BANNER(w), FALSE);
+		w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_refresh_lvfs"));
+		adw_banner_set_revealed(ADW_BANNER(w), FALSE);
 	}
+
+	/* this might have changed */
+	gfu_main_build_report_devices_payload(self);
 }
 
 static gint
@@ -652,6 +712,192 @@ gfu_main_update_devices_activate_child_cb(gpointer user_data)
 	return G_SOURCE_REMOVE;
 }
 
+#if FWUPD_CHECK_VERSION(1, 9, 20)
+static void
+gfu_main_report_device_list_upload_report_cb(GObject *source, GAsyncResult *res, gpointer user_data)
+{
+	GfuMain *self = (GfuMain *)user_data;
+	g_autofree gchar *uri = NULL;
+	g_autoptr(GError) error = NULL;
+
+	/* no point keeping this now */
+	g_clear_pointer(&self->report_devices_payload, g_free);
+
+	uri = fwupd_client_upload_report_finish(FWUPD_CLIENT(source), res, &error);
+	if (uri == NULL) {
+		/* TRANSLATORS: the LVFS remote could not be enabled */
+		gfu_main_error_dialog(self, _("Failed to send report to the LVFS"), error->message);
+		return;
+	}
+}
+
+static void
+gfu_main_report_device_list_dialog_response_cb(AdwMessageDialog *dialog,
+					       const gchar *response,
+					       GfuMain *self)
+{
+	if (g_strcmp0(response, "upload") != 0)
+		return;
+	fwupd_client_upload_report_async(self->client,
+					 fwupd_remote_get_report_uri(self->remote_lvfs),
+					 self->report_devices_payload,
+					 NULL, /* signature */
+					 FWUPD_CLIENT_UPLOAD_FLAG_ALWAYS_MULTIPART,
+					 self->cancellable,
+					 gfu_main_report_device_list_upload_report_cb,
+					 self);
+}
+
+static void
+gfu_main_button_copy_text_clicked_cb(GtkButton *button, GtkTextView *text_view)
+{
+	GdkClipboard *clipboard = gtk_widget_get_clipboard(GTK_WIDGET(text_view));
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer(text_view);
+	GtkTextIter start, end;
+	g_autofree gchar *text = NULL;
+
+	gtk_text_buffer_get_bounds(buffer, &start, &end);
+	text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+	gdk_clipboard_set_text(clipboard, text);
+}
+
+static void
+gfu_main_report_device_list_dialog(GfuMain *self)
+{
+	GtkTextBuffer *text_buffer;
+	GtkWidget *dialog;
+	GtkWidget *hbox;
+	GtkWidget *label_dl;
+	GtkWidget *scrolled_window;
+	GtkWidget *text_view;
+	GtkWidget *vbox;
+	GtkWindow *window;
+	GtkWidget *button_copy;
+	g_autoptr(GError) error_local = NULL;
+
+	/* show UI */
+	window = GTK_WINDOW(gtk_builder_get_object(self->builder, "dialog_main"));
+	/* TRANSLATORS: dialog title */
+	dialog = adw_message_dialog_new(window, _("Upload Device List?"), NULL);
+	adw_message_dialog_format_body(
+	    ADW_MESSAGE_DIALOG(dialog),
+	    /* TRANSLATORS: dialog body, %1 is typically "Linux Vendor Firmware Service"
+	     *  and %2 is a URL to the privacy policy */
+	    _("Help to improve the %s by sending the list of current devices? The list is "
+	      "anonymous and does not include personally identifiable information  — see the <a "
+	      "href=\"%s\">privacy policy</a> for more information."),
+	    fwupd_remote_get_title(self->remote_lvfs),
+	    GFU_LVFS_PRIVACY_POLICY_URL);
+	adw_message_dialog_add_responses(ADW_MESSAGE_DIALOG(dialog),
+					 "cancel",
+					 /* TRANSLATORS: button text */
+					 _("_Cancel"),
+					 "upload",
+					 /* TRANSLATORS: button text */
+					 _("_Upload"),
+					 NULL);
+	adw_message_dialog_set_body_use_markup(ADW_MESSAGE_DIALOG(dialog), TRUE);
+	adw_message_dialog_set_response_appearance(ADW_MESSAGE_DIALOG(dialog),
+						   "upload",
+						   ADW_RESPONSE_SUGGESTED);
+	adw_message_dialog_set_default_response(ADW_MESSAGE_DIALOG(dialog), "upload");
+	adw_message_dialog_set_close_response(ADW_MESSAGE_DIALOG(dialog), "cancel");
+
+	/* TRANSLATORS: header before the JSON listing */
+	label_dl = gtk_label_new(_("Device List"));
+	gtk_widget_set_halign(label_dl, GTK_ALIGN_START);
+	gtk_widget_add_css_class(label_dl, "title-4");
+	gtk_widget_set_hexpand(label_dl, TRUE);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 9);
+	gtk_box_append(GTK_BOX(hbox), label_dl);
+
+	/* json blob */
+	text_buffer = gtk_text_buffer_new(NULL);
+	gtk_text_buffer_set_text(text_buffer, self->report_devices_payload, -1);
+	text_view = gtk_text_view_new_with_buffer(text_buffer);
+	gtk_text_view_set_monospace(GTK_TEXT_VIEW(text_view), TRUE);
+	gtk_text_view_set_top_margin(GTK_TEXT_VIEW(text_view), 9);
+	gtk_text_view_set_bottom_margin(GTK_TEXT_VIEW(text_view), 9);
+	gtk_text_view_set_left_margin(GTK_TEXT_VIEW(text_view), 9);
+	gtk_text_view_set_right_margin(GTK_TEXT_VIEW(text_view), 9);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_CHAR);
+	gtk_widget_set_margin_end(text_view, 9);
+	gtk_widget_add_css_class(text_view, "inline");
+
+	/* make scrollable */
+	scrolled_window = gtk_scrolled_window_new();
+	gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(scrolled_window), text_view);
+	gtk_scrolled_window_set_propagate_natural_width(GTK_SCROLLED_WINDOW(scrolled_window), TRUE);
+	gtk_scrolled_window_set_propagate_natural_height(GTK_SCROLLED_WINDOW(scrolled_window),
+							 TRUE);
+	gtk_widget_add_css_class(scrolled_window, "card");
+
+	/* button to copy the contents to the clipboard */
+	button_copy = gtk_button_new_from_icon_name("edit-copy-symbolic");
+	g_object_set(button_copy, "focus-on-click", FALSE, NULL);
+	gtk_widget_set_halign(button_copy, GTK_ALIGN_END);
+	gtk_widget_add_css_class(button_copy, "flat");
+	gtk_box_append(GTK_BOX(hbox), button_copy);
+	g_signal_connect(button_copy,
+			 "clicked",
+			 G_CALLBACK(gfu_main_button_copy_text_clicked_cb),
+			 text_view);
+
+	/* add extra section */
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 9);
+	gtk_box_append(GTK_BOX(vbox), hbox);
+	gtk_box_append(GTK_BOX(vbox), scrolled_window);
+	adw_message_dialog_set_extra_child(ADW_MESSAGE_DIALOG(dialog), vbox);
+
+	/* show */
+	g_signal_connect(dialog,
+			 "response",
+			 G_CALLBACK(gfu_main_report_device_list_dialog_response_cb),
+			 self);
+	gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
+}
+
+static void
+gfu_main_report_check_device_list(GfuMain *self)
+{
+	GtkWidget *w;
+
+	/* sanity check */
+	if (self->remote_lvfs == NULL) {
+		g_debug("no LVFS remote");
+		return;
+	}
+	if (fwupd_remote_needs_refresh(self->remote_lvfs)) {
+		g_debug("LVFS remote needs refresh");
+		return;
+	}
+	if (self->report_devices_payload == NULL) {
+		g_debug("no report metadata");
+		return;
+	}
+
+	/* show banner */
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_device_list"));
+	adw_banner_set_revealed(ADW_BANNER(w), TRUE);
+}
+
+static gboolean
+gfu_main_report_device_list_cb(gpointer user_data)
+{
+	GfuMain *self = (GfuMain *)user_data;
+	gint64 interval = GFU_MAIN_REPORT_DEVICE_LIST_INTERVAL_DELAY * 24 * 60 * 60;
+	gint64 seconds = g_get_real_time() / G_USEC_PER_SEC;
+	gint64 seconds_last = g_settings_get_int64(self->settings, "report-device-list-timestamp");
+
+	if (g_getenv("GNOME_FIRMWARE_FORCE_DEVICE_LIST") != NULL ||
+	    seconds > seconds_last + interval) {
+		g_settings_set_int64(self->settings, "report-device-list-timestamp", seconds);
+		gfu_main_report_check_device_list(self);
+	}
+	return G_SOURCE_REMOVE;
+}
+#endif
+
 static void
 gfu_main_update_devices_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 {
@@ -660,26 +906,27 @@ gfu_main_update_devices_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 	AdwNavigationSplitView *split_view =
 	    ADW_NAVIGATION_SPLIT_VIEW(gtk_builder_get_object(self->builder, "split_view"));
 	g_autoptr(GError) error = NULL;
-	g_autoptr(GPtrArray) devices = NULL;
 
 	/* stop the spinner now */
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "spinner_loading"));
 	gtk_spinner_stop(GTK_SPINNER(w));
 
-	devices = fwupd_client_get_devices_finish(FWUPD_CLIENT(source), res, &error);
-	if (devices == NULL) {
+	if (self->devices != NULL)
+		g_ptr_array_unref(self->devices);
+	self->devices = fwupd_client_get_devices_finish(FWUPD_CLIENT(source), res, &error);
+	if (self->devices == NULL) {
 		/* TRANSLATORS: fwupd refused us data */
 		gfu_main_error_dialog(self, _("Failed to load device list"), error->message);
 		return;
 	}
 
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "device_metadata"));
-	gtk_widget_set_visible(GTK_WIDGET(w), devices->len > 0);
+	gtk_widget_set_visible(GTK_WIDGET(w), self->devices->len > 0);
 
 	/* create a row in the listbox for each updatable device */
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "listbox_devices"));
-	for (guint i = 0; i < devices->len; i++) {
-		FwupdDevice *device = g_ptr_array_index(devices, i);
+	for (guint i = 0; i < self->devices->len; i++) {
+		FwupdDevice *device = g_ptr_array_index(self->devices, i);
 		GtkWidget *l;
 
 		/* skip devices that don't have a firmware version */
@@ -709,30 +956,63 @@ gfu_main_update_devices_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 	g_idle_add(gfu_main_update_devices_activate_child_cb, self);
 }
 
+static void
+gfu_main_release_row_activated_cb(AdwActionRow *row, GfuMain *self)
+{
+	FwupdRelease *release = gfu_release_row_get_release(GFU_RELEASE_ROW(row));
+	GtkWidget *w;
+	GtkWidget *split_view = GTK_WIDGET(gtk_builder_get_object(self->builder, "split_view"));
+
+	g_set_object(&self->release, release);
+	gfu_main_refresh_release(self);
+
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "release_window_title"));
+	adw_window_title_set_title(ADW_WINDOW_TITLE(w), fwupd_release_get_version(release));
+
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "window_release"));
+	gtk_window_set_default_size(GTK_WINDOW(w),
+				    gtk_widget_get_width(split_view) * 85 / 100,
+				    gtk_widget_get_height(split_view) * 60 / 100);
+	gtk_window_present(GTK_WINDOW(w));
+}
+
 static void
 gfu_main_update_releases(GfuMain *self, GPtrArray *releases)
 {
 	GtkWidget *w;
+	GPtrArray *children;
 
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "group_releases"));
-	gtk_widget_set_visible(w, releases != NULL);
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "group_releases_none"));
 	gtk_widget_set_visible(w, releases == NULL);
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "group_releases"));
+	gtk_widget_set_visible(w, releases != NULL);
 
 	/* nothing to do */
 	if (releases == NULL)
 		return;
 
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "listbox_releases"));
+	/* we have to do this hack as adw_preferences_group_remove_all() does not exist */
+	children = gfu_main_get_secret_children(w);
+	for (guint i = 0; i < children->len; i++) {
+		GtkWidget *row = g_ptr_array_index(children, i);
+		adw_preferences_group_remove(ADW_PREFERENCES_GROUP(w), row);
+	}
+	g_ptr_array_set_size(children, 0);
+
 	for (guint i = 0; i < releases->len; i++) {
 		FwupdRelease *release = g_ptr_array_index(releases, i);
-		GtkWidget *l = gfu_release_row_new(self->device, release);
+		GtkWidget *row = gfu_release_row_new(self->device, release);
 		g_debug("adding release %s", fwupd_release_get_version(release));
-		gtk_list_box_insert(GTK_LIST_BOX(w), l, -1);
-		g_signal_connect(l,
+		adw_preferences_group_add(ADW_PREFERENCES_GROUP(w), row);
+		g_ptr_array_add(children, row);
+		g_signal_connect(row,
 				 "button-clicked",
 				 G_CALLBACK(gfu_main_release_row_button_clicked_cb),
 				 self);
+		g_signal_connect(row,
+				 "activated",
+				 G_CALLBACK(gfu_main_release_row_activated_cb),
+				 self);
 	}
 }
 
@@ -802,28 +1082,18 @@ gfu_main_refresh_remote_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 }
 
 static void
-gfu_main_get_lvfs_remote_cb(GObject *source, GAsyncResult *res, gpointer user_data)
+gfu_main_refresh_remote(GfuMain *self)
 {
-	GfuMain *self = (GfuMain *)user_data;
-	g_autoptr(FwupdRemote) remote = NULL;
-	g_autoptr(GError) error = NULL;
-
-	remote = fwupd_client_get_remote_by_id_finish(FWUPD_CLIENT(source), res, &error);
-	if (remote == NULL) {
-		/* TRANSLATORS: the LVFS remote was not found on the system */
-		gfu_main_error_dialog(self, _("Failed to find LVFS"), error->message);
-		return;
-	}
 #if FWUPD_CHECK_VERSION(2, 0, 0)
 	fwupd_client_refresh_remote_async(self->client,
-					  remote,
+					  self->remote_lvfs,
 					  FWUPD_CLIENT_DOWNLOAD_FLAG_NONE,
 					  self->cancellable,
 					  gfu_main_refresh_remote_cb,
 					  self);
 #else
 	fwupd_client_refresh_remote_async(self->client,
-					  remote,
+					  self->remote_lvfs,
 					  self->cancellable,
 					  gfu_main_refresh_remote_cb,
 					  self);
@@ -841,32 +1111,7 @@ gfu_main_modify_remote_cb(GObject *source, GAsyncResult *res, gpointer user_data
 		gfu_main_error_dialog(self, _("Failed to enable LVFS"), error->message);
 		return;
 	}
-
-	/* refresh the newly-enabled remote */
-	fwupd_client_get_remote_by_id_async(self->client,
-					    "lvfs",
-					    self->cancellable,
-					    gfu_main_get_lvfs_remote_cb,
-					    self);
-}
-
-static void
-gfu_main_enable_lvfs_cb(GtkWidget *widget, GfuMain *self)
-{
-	GtkWidget *w;
-
-	/* hide notification */
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "infobar_enable_lvfs"));
-	gtk_info_bar_set_revealed(GTK_INFO_BAR(w), FALSE);
-
-	/* enable remote */
-	fwupd_client_modify_remote_async(self->client,
-					 "lvfs",
-					 "Enabled",
-					 "true",
-					 self->cancellable,
-					 gfu_main_modify_remote_cb,
-					 self);
+	gfu_main_refresh_remote(self);
 }
 
 static void
@@ -884,11 +1129,11 @@ gfu_main_get_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 	}
 	for (guint i = 0; i < remotes->len; i++) {
 		FwupdRemote *remote = g_ptr_array_index(remotes, i);
-#if FWUPD_CHECK_VERSION(2, 0, 0)
 		if (!fwupd_remote_has_flag(remote, FWUPD_REMOTE_FLAG_ENABLED))
 			continue;
 		if (fwupd_remote_get_kind(remote) != FWUPD_REMOTE_KIND_DOWNLOAD)
 			continue;
+#if FWUPD_CHECK_VERSION(2, 0, 0)
 		/* should this be a single async action? */
 		fwupd_client_refresh_remote_async(self->client,
 						  remote,
@@ -897,11 +1142,6 @@ gfu_main_get_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 						  gfu_main_refresh_remote_cb,
 						  self);
 #else
-		if (!fwupd_remote_get_enabled(remote))
-			continue;
-		if (fwupd_remote_get_kind(remote) != FWUPD_REMOTE_KIND_DOWNLOAD)
-			continue;
-		/* should this be a single async action? */
 		fwupd_client_refresh_remote_async(self->client,
 						  remote,
 						  self->cancellable,
@@ -911,6 +1151,28 @@ gfu_main_get_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 	}
 }
 
+#if FWUPD_CHECK_VERSION(1, 9, 20)
+static void
+gfu_main_activate_upload_report_devices(GSimpleAction *simple,
+					GVariant *parameter,
+					gpointer user_data)
+{
+	GfuMain *self = (GfuMain *)user_data;
+
+	/* is this possible */
+	if (self->report_devices_payload == NULL) {
+		gfu_main_error_dialog(self,
+				      /* TRANSLATORS: dialog title */
+				      _("No report to upload"),
+				      /* TRANSLATORS: dialog title */
+				      _("There are no unsupported devices detected."));
+		return;
+	}
+
+	gfu_main_report_device_list_dialog(self);
+}
+#endif
+
 static void
 gfu_main_activate_refresh_metadata(GSimpleAction *simple, GVariant *parameter, gpointer user_data)
 {
@@ -921,6 +1183,15 @@ gfu_main_activate_refresh_metadata(GSimpleAction *simple, GVariant *parameter, g
 				       self);
 }
 
+#if FWUPD_CHECK_VERSION(1, 9, 17)
+static void
+gfu_main_request_invalidate_cb(FwupdRequest *request, GtkWidget *dialog)
+{
+	g_debug("request invalidated");
+	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+}
+#endif
+
 static void
 gfu_main_show_request_real(GfuMain *self, FwupdRequest *request, GdkPixbuf *pixbuf)
 {
@@ -956,6 +1227,9 @@ gfu_main_show_request_real(GfuMain *self, FwupdRequest *request, GdkPixbuf *pixb
 					    gdk_pixbuf_get_height(pixbuf));
 		gtk_box_append(GTK_BOX(box), image);
 	}
+#if FWUPD_CHECK_VERSION(1, 9, 17)
+	g_signal_connect(request, "invalidate", G_CALLBACK(gfu_main_request_invalidate_cb), dialog);
+#endif
 
 	gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 }
@@ -1507,9 +1781,13 @@ gfu_main_quit_activated_cb(GSimpleAction *action, GVariant *parameter, gpointer
 	g_application_quit(G_APPLICATION(self->application));
 }
 
-static GActionEntry actions[] = {{"about", gfu_main_about_activated_cb, NULL, NULL, NULL},
-				 {"refresh", gfu_main_activate_refresh_metadata, NULL, NULL, NULL},
-				 {"quit", gfu_main_quit_activated_cb, NULL, NULL, NULL}};
+static GActionEntry actions[] = {
+    {"about", gfu_main_about_activated_cb, NULL, NULL, NULL},
+    {"refresh", gfu_main_activate_refresh_metadata, NULL, NULL, NULL},
+#if FWUPD_CHECK_VERSION(1, 9, 20)
+    {"upload-report-devices", gfu_main_activate_upload_report_devices, NULL, NULL, NULL},
+#endif
+    {"quit", gfu_main_quit_activated_cb, NULL, NULL, NULL}};
 
 static void
 gfu_main_refresh_release(GfuMain *self)
@@ -1519,6 +1797,8 @@ gfu_main_refresh_release(GfuMain *self)
 	GPtrArray *checks = NULL;
 	GPtrArray *issues = NULL;
 	g_autofree gchar *desc = NULL;
+	g_autofree gchar *install_duration = NULL;
+	g_autofree gchar *release_flags = NULL;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GString) attr = g_string_new(NULL);
 
@@ -1621,41 +1901,18 @@ gfu_main_refresh_release(GfuMain *self)
 				   "label_release_license",
 				   fwupd_release_get_license(self->release));
 	}
-	gfu_main_set_label(
-	    self,
-	    "label_release_flags",
-	    gfu_common_release_flags_to_strings(fwupd_release_get_flags(self->release)));
+	release_flags = gfu_common_release_flags_to_strings(fwupd_release_get_flags(self->release));
+	gfu_main_set_label(self, "label_release_flags", release_flags);
 
-	gfu_main_set_label(
-	    self,
-	    "label_release_install_duration",
-	    gfu_common_seconds_to_string(fwupd_release_get_install_duration(self->release)));
+	install_duration =
+	    gfu_common_seconds_to_string(fwupd_release_get_install_duration(self->release));
+	gfu_main_set_label(self, "label_release_install_duration", install_duration);
 
 	gfu_main_set_label(self,
 			   "label_release_update_message",
 			   fwupd_release_get_update_message(self->release));
 }
 
-static void
-gfu_main_release_row_selected_cb(GtkListBox *box, GtkListBoxRow *row, GfuMain *self)
-{
-	FwupdRelease *release = gfu_release_row_get_release(GFU_RELEASE_ROW(row));
-	GtkWidget *w;
-	GtkWidget *split_view = GTK_WIDGET(gtk_builder_get_object(self->builder, "split_view"));
-
-	g_set_object(&self->release, release);
-	gfu_main_refresh_release(self);
-
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "release_window_title"));
-	adw_window_title_set_title(ADW_WINDOW_TITLE(w), fwupd_release_get_version(release));
-
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "window_release"));
-	gtk_window_set_default_size(GTK_WINDOW(w),
-				    gtk_widget_get_width(split_view) * 85 / 100,
-				    gtk_widget_get_height(split_view) * 60 / 100);
-	gtk_window_present(GTK_WINDOW(w));
-}
-
 static void
 gfu_main_refresh_releases(GfuMain *self)
 {
@@ -1663,8 +1920,6 @@ gfu_main_refresh_releases(GfuMain *self)
 
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "group_releases"));
 	gtk_widget_set_visible(w, FALSE);
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "listbox_releases"));
-	gfu_main_list_box_remove_all(GTK_LIST_BOX(w));
 
 	/* async call for releases */
 	if (self->device != NULL &&
@@ -1697,6 +1952,25 @@ gfu_main_device_row_selected_cb(GtkListBox *box, GtkListBoxRow *row, GfuMain *se
 	gfu_main_refresh_device(self);
 }
 
+static void
+gfu_main_get_report_metadata_cb(GObject *source, GAsyncResult *res, gpointer user_data)
+{
+	GfuMain *self = (GfuMain *)user_data;
+	g_autoptr(GError) error = NULL;
+
+	if (self->report_metadata != NULL)
+		g_hash_table_unref(self->report_metadata);
+	self->report_metadata =
+	    fwupd_client_get_report_metadata_finish(FWUPD_CLIENT(source), res, &error);
+	if (self->report_metadata == NULL) {
+		g_warning("failed to get report metadata: %s", error->message);
+		return;
+	}
+
+	/* this will be valid now */
+	gfu_main_build_report_devices_payload(self);
+}
+
 static void
 gfu_main_set_feature_flags_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 {
@@ -1719,6 +1993,10 @@ gfu_main_set_feature_flags_cb(GObject *source, GAsyncResult *res, gpointer user_
 				       self->cancellable,
 				       gfu_main_update_remotes_cb,
 				       self);
+	fwupd_client_get_report_metadata_async(self->client,
+					       self->cancellable,
+					       gfu_main_get_report_metadata_cb,
+					       self);
 }
 
 static void
@@ -1788,16 +2066,46 @@ gfu_main_device_unlock_cb(GtkWidget *widget, gboolean state, GfuMain *self)
 }
 
 static void
-gfu_main_infobar_close_cb(GtkInfoBar *infobar, gpointer user_data)
+gfu_main_banner_refresh_lvfs_clicked_cb(AdwBanner *banner, gpointer user_data)
 {
-	gtk_info_bar_set_revealed(infobar, FALSE);
+	GfuMain *self = (GfuMain *)user_data;
+
+	/* hide banner */
+	adw_banner_set_revealed(banner, FALSE);
+
+	gfu_main_refresh_remote(self);
 }
 
 static void
-gfu_main_infobar_response_cb(GtkInfoBar *infobar, gint response_id, gpointer user_data)
+gfu_main_banner_enable_lvfs_clicked_cb(AdwBanner *banner, gpointer user_data)
 {
-	if (response_id == GTK_RESPONSE_CLOSE)
-		gtk_info_bar_set_revealed(infobar, FALSE);
+	GfuMain *self = (GfuMain *)user_data;
+
+	/* hide banner */
+	adw_banner_set_revealed(banner, FALSE);
+
+	/* enable remote */
+	fwupd_client_modify_remote_async(self->client,
+					 "lvfs",
+					 "Enabled",
+					 "true",
+					 self->cancellable,
+					 gfu_main_modify_remote_cb,
+					 self);
+}
+
+static void
+gfu_main_banner_device_list_clicked_cb(AdwBanner *banner, gpointer user_data)
+{
+#if FWUPD_CHECK_VERSION(1, 9, 20)
+	GfuMain *self = (GfuMain *)user_data;
+	GtkWidget *w;
+
+	gfu_main_report_device_list_dialog(self);
+
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_device_list"));
+	adw_banner_set_revealed(ADW_BANNER(w), FALSE);
+#endif
 }
 
 static void
@@ -1822,13 +2130,18 @@ gfu_main_client_connect_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 	    self->client,
 	    FWUPD_FEATURE_FLAG_FDE_WARNING | FWUPD_FEATURE_FLAG_SWITCH_BRANCH |
 		FWUPD_FEATURE_FLAG_SHOW_PROBLEMS | FWUPD_FEATURE_FLAG_REQUESTS |
-#if FWUPD_CHECK_VERSION(1, 9, 8)
-		FWUPD_FEATURE_FLAG_REQUESTS_NON_GENERIC |
-#endif
-		FWUPD_FEATURE_FLAG_ALLOW_AUTHENTICATION | FWUPD_FEATURE_FLAG_UPDATE_ACTION,
+		FWUPD_FEATURE_FLAG_REQUESTS_NON_GENERIC | FWUPD_FEATURE_FLAG_ALLOW_AUTHENTICATION |
+		FWUPD_FEATURE_FLAG_UPDATE_ACTION,
 	    self->cancellable,
 	    gfu_main_set_feature_flags_cb,
 	    self);
+
+#if FWUPD_CHECK_VERSION(1, 9, 20)
+	/* wait for the user to settle in */
+	g_timeout_add_seconds(GFU_MAIN_REPORT_DEVICE_LIST_INITIAL_DELAY,
+			      gfu_main_report_device_list_cb,
+			      self);
+#endif
 }
 
 static void
@@ -1836,6 +2149,7 @@ gfu_main_startup_cb(GApplication *application, GfuMain *self)
 {
 	GtkWidget *w;
 	GtkWidget *main_window;
+	g_autoptr(GtkCssProvider) css_provider = gtk_css_provider_new();
 
 	/* add application menu items */
 	g_action_map_add_action_entries(G_ACTION_MAP(application),
@@ -1849,6 +2163,7 @@ gfu_main_startup_cb(GApplication *application, GfuMain *self)
 	main_window = GTK_WIDGET(gtk_builder_get_object(self->builder, "dialog_main"));
 	gtk_application_add_window(GTK_APPLICATION(self->application), GTK_WINDOW(main_window));
 	gtk_window_set_default_size(GTK_WINDOW(main_window), 1024, 600);
+	gtk_window_set_icon_name(GTK_WINDOW(main_window), "org.gnome.Firmware");
 
 	/* hide window first so that the dialogue resizes itself without redrawing */
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "stack_main"));
@@ -1861,16 +2176,23 @@ gfu_main_startup_cb(GApplication *application, GfuMain *self)
 	g_signal_connect(w, "clicked", G_CALLBACK(gfu_main_device_verify_clicked_cb), self);
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "button_verify_update"));
 	g_signal_connect(w, "clicked", G_CALLBACK(gfu_main_device_verify_update_cb), self);
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "button_infobar_enable_lvfs"));
-	g_signal_connect(w, "clicked", G_CALLBACK(gfu_main_enable_lvfs_cb), self);
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "listbox_devices"));
 	g_signal_connect(w, "row-activated", G_CALLBACK(gfu_main_device_row_selected_cb), self);
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "listbox_releases"));
-	g_signal_connect(w, "row-activated", G_CALLBACK(gfu_main_release_row_selected_cb), self);
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "infobar_enable_lvfs"));
-	g_signal_connect(w, "close", G_CALLBACK(gfu_main_infobar_close_cb), self);
-	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "infobar_enable_lvfs"));
-	g_signal_connect(w, "response", G_CALLBACK(gfu_main_infobar_response_cb), self);
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_enable_lvfs"));
+	g_signal_connect(w,
+			 "button-clicked",
+			 G_CALLBACK(gfu_main_banner_enable_lvfs_clicked_cb),
+			 self);
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_refresh_lvfs"));
+	g_signal_connect(w,
+			 "button-clicked",
+			 G_CALLBACK(gfu_main_banner_refresh_lvfs_clicked_cb),
+			 self);
+	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "banner_device_list"));
+	g_signal_connect(w,
+			 "button-clicked",
+			 G_CALLBACK(gfu_main_banner_device_list_clicked_cb),
+			 self);
 
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "window_release"));
 	g_signal_connect(ADW_WINDOW(w), "close-request", G_CALLBACK(gtk_widget_hide), self);
@@ -1883,6 +2205,17 @@ gfu_main_startup_cb(GApplication *application, GfuMain *self)
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "spinner_loading"));
 	gtk_spinner_start(GTK_SPINNER(w));
 
+	/* required for https://gitlab.gnome.org/GNOME/libadwaita/-/merge_requests/1118 */
+	gtk_css_provider_load_from_string(css_provider,
+					  "textview.inline,\n"
+					  "textview.inline text {\n"
+					  "background: none;\n"
+					  "color: inherit;\n"
+					  "}\n");
+	gtk_style_context_add_provider_for_display(gdk_display_get_default(),
+						   GTK_STYLE_PROVIDER(css_provider),
+						   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
 	/* show main UI */
 	gtk_widget_show(main_window);
 	gfu_main_update_progress_bar(self);
@@ -1926,6 +2259,7 @@ gfu_main_device_request_cb(FwupdClient *client, FwupdRequest *request, GfuMain *
 static void
 gfu_main_free(GfuMain *self)
 {
+	g_free(self->report_devices_payload);
 	if (self->refresh_id != 0)
 		g_source_remove(self->refresh_id);
 	if (self->builder != NULL)
@@ -1940,10 +2274,18 @@ gfu_main_free(GfuMain *self)
 		g_object_unref(self->client);
 	if (self->application != NULL)
 		g_object_unref(self->application);
+	if (self->settings != NULL)
+		g_object_unref(self->settings);
 	if (self->dialogs != NULL)
 		g_ptr_array_unref(self->dialogs);
 	if (self->post_requests != NULL)
 		g_ptr_array_unref(self->post_requests);
+	if (self->devices != NULL)
+		g_ptr_array_unref(self->devices);
+	if (self->report_metadata != NULL)
+		g_hash_table_unref(self->report_metadata);
+	if (self->remote_lvfs != NULL)
+		g_object_unref(self->remote_lvfs);
 	g_free(self);
 }
 
@@ -2010,6 +2352,7 @@ main(int argc, char **argv)
 	self->post_requests = g_ptr_array_new_with_free_func((GDestroyNotify)g_object_unref);
 
 	/* ensure single instance */
+	self->settings = g_settings_new("org.gnome.Firmware");
 	self->application = adw_application_new("org.gnome.Firmware", G_APPLICATION_DEFAULT_FLAGS);
 	g_signal_connect(self->application, "startup", G_CALLBACK(gfu_main_startup_cb), self);
 	g_signal_connect(self->application, "activate", G_CALLBACK(gfu_main_activate_cb), self);
