@@ -6,17 +6,17 @@
 
 #pragma once
 
+#include <adwaita.h>
 #include <fwupd.h>
-#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
 #define GFU_TYPE_RELEASE_ROW (gfu_release_row_get_type())
 
-G_DECLARE_DERIVABLE_TYPE(GfuReleaseRow, gfu_release_row, GFU, RELEASE_ROW, GtkListBoxRow)
+G_DECLARE_DERIVABLE_TYPE(GfuReleaseRow, gfu_release_row, GFU, RELEASE_ROW, AdwActionRow)
 
 struct _GfuReleaseRowClass {
-	GtkListBoxRowClass parent_class;
+	AdwActionRowClass parent_class;
 	void (*button_clicked)(GfuReleaseRow *self);
 };
 
